unit Unit_Test;

interface

uses Unit_Ppal, Unit_Solv;

function GrilleOK:boolean;
procedure ChercherErreurs;
function GrilleResolue:boolean;
function ZoneComplete(iZon:TiZon):boolean;
function CandidatUniqueZone(iZon:TiZon;v:TVal):TiCaz;
function CandidatUniqueInterZone(r:TiZon;z:TiZon;v:TVal):TiCaz;
function CandidatDansInterZone(r:TiZon;cl:Tizon;cnd:TVal):boolean;
function EluDansZone(iZon:TiZon;v:TVal):TiCaz;
function CasesContenantCdt(iZon:TiZon;v:TVal):TEnsCaz;
function NbMinCdt:byte;

implementation

uses SysUtils;

function GrilleOK;
//teste si la grille est valide
var
  iZon:TiZon;
  c1,c2:0..8;
  EnsV:TEnsVal;
begin
  result:=true;
  for iZon:=0 to 26 do
  //parcourir les 27 zones
  for c1:=0 to 7 do
  //parcourir les 7 premires cases de chaque zone
  begin
    EnsV:=Caz^[IndCaz[iZon,c1]].Cdt;
    if NbVal(EnsV)=1
    //tester si 2 fois le mme singleton dans la mme zone
    then for c2:=c1+1 to 8 do
         if Caz^[IndCaz[iZon,c2]].Cdt=EnsV
         then begin
                result:=false;
                exit;
              end;
  end;
end;

procedure ChercherErreurs;
// la diffrence de la function GrilleOK ci-dessus, qui est arrte ds la 1re erreur trouve
//ici on fait un contrle exhaustif de toutes les cases pour visualiser toutes les erreurs
var
  iZon:TiZon;
  iCaz:TiCaz;
  c1,c2:0..8;
  EnsV:TEnsVal;
begin
  for iCaz:=0 to 80 do
  Caz^[iCaz].Err:=false;
  for iZon:=0 to 26 do
  for c1:=0 to 7 do
  begin
    EnsV:=Caz^[IndCaz[iZon,c1]].Cdt;
    if NbVal(EnsV)=1
    //tester si 2 fois le mme singleton dans la mme zone
    then for c2:=c1+1 to 8 do
         if Caz^[IndCaz[iZon,c2]].Cdt=EnsV
         then begin
                Caz^[IndCaz[iZon,c1]].Err:=true;
                Caz^[IndCaz[iZon,c2]].Err:=true;
              end;
  end;
  MaGrille.Repaint;
end;

function GrilleResolue:boolean;
//la grille est rsolue si toutes les cases n'ont qu'1 candidat (ou 1 monarque)
var
  iCaz:TiCaz;
begin
  result:=true;
  for iCaz:=0 to 80 do
  if not Caz^[iCaz].Monarq
  then if NbVal(Caz^[iCaz].Cdt)>1
       then begin
              result:=false;
              exit;
            end;
end;

function ZoneComplete(iZon:TiZon):boolean;
//teste si toutes les valeurs d'une zone sont rvles ou trouves
var
  n:0..8;
begin
  result:=true;
  for n:=0 to 8 do
  if not Caz^[IndCaz[iZon,n]].Monarq
  then if NbVal(Caz^[IndCaz[iZon,n]].Cdt)>1
       then begin
              result:=false;
              exit;
            end;
end;

function CandidatUniqueZone(iZon:TiZon;v:TVal):TiCaz;
//renvoie l'indice 0  80 de la case contenant le candidat unique v de la zone iZon
//(le candidat n'est ligible que dans 1 case de la zone)
//renvoie -1 s'il n'existe pas
var
  c:0..8;
  EnsValTemp,EnsValTest:TEnsVal;
begin
  result:=-1;
  EnsValTemp:=[];
  EnsValTest:=[1..9];
  for c:=0 to 8 do //parcourir les 9 cases de la zone
  begin
    //enlever de EnsValTest les candidats dj prsents dans les cases prcdentes
    EnsValTest:=EnsValTest-(EnsValTemp*Caz^[IndCaz[iZon,c]].Cdt);
    //enlever de EnsValTest la valeur de la case si lu
    if NbVal(Caz^[IndCaz[iZon,c]].Cdt)=1
    then EnsValTest:=EnsValTest-Caz^[IndCaz[iZon,c]].Cdt;
    //runir dans EnsValTemp tous les candidats dj vus
    EnsValTemp:=EnsValTemp+Caz^[IndCaz[iZon,c]].Cdt;
  end;
  // ce stade, EnsValTest n'inclut plus les candidats prsents dans 2 cases ou + de la zone
  if(EnsValTest<>[])
  and(v in EnsValTest)
  then for c:=0 to 8 do
       begin
         if v in Caz^[IndCaz[iZon,c]].Cdt
         then result:=IndCaz[iZon,c];
       end;
end;

function CandidatUniqueInterZone(r:TiZon;z:TiZon;v:Tval):TiCaz;
//renvoie l'indice 0  80 de la case contenant le candidat unique v
//de l'interzone forme entre la rgion r et la colonne ou ligne z
//renvoie -1 si cette case n'existe pas
var
  c:0..8;
  EnsValTemp,EnsValTest:TEnsVal;
begin
  result:=-1;
  EnsValTemp:=[];
  EnsValTest:=[1..9];
  for c:=0 to 8 do //parcourir les 9 cases de la rgion
  if(z in Caz^[IndCaz[r,c]].Zon) //ne retenir que les cases de l'interzone
  then begin
         //enlever de EnsValTest les candidats dj prsents dans les cases prcdentes
         EnsValTest:=EnsValTest-(EnsValTemp*Caz^[IndCaz[r,c]].Cdt);
         //enlever de EnsValTest la valeur de la case si lu
         if NbVal(Caz^[IndCaz[z,c]].Cdt)=1
         then EnsValTest:=EnsValTest-Caz^[IndCaz[r,c]].Cdt;
         //runir dans EnsValTemp tous les candidats dj vus
         EnsValTemp:=EnsValTemp+Caz^[IndCaz[r,c]].Cdt
       end;
  // ce stade, EnsValTest n'inclut plus les candidats prsents dans 2 cases ou + de l'interzone
  if v in EnsValTest
  then for c:=0 to 8 do
       begin
         if(z in Caz^[IndCaz[r,c]].Zon)
         and(v in Caz^[IndCaz[r,c]].Cdt)
         then result:=IndCaz[r,c];
       end;
end;

function CandidatDansInterZone(r:TiZon;cl:Tizon;cnd:TVal):boolean;
//renvoie true si le candidat cnd est prsent dans l'interzone forme
//entre la rgion r et la colonne/ligne cl
var
  c:0..8;
begin
  result:=false;
  for c:=0 to 8 do
  case cl of
  0..8:  if(cnd in Caz^[IndCaz[r,c]].Cdt)
         and(ZoneCaz[IndCaz[r,c],0]=cl)
         then begin
                result:=true;
                exit;
              end;
  9..17: if(cnd in Caz^[IndCaz[r,c]].Cdt)
         and(ZoneCaz[IndCaz[r,c],1]=cl)
         then begin
                result:=true;
                exit;
              end;
  end;
end;

function EluDansZone(iZon:TiZon;v:TVal):TiCaz;
//renvoie l'indice de case contenant la valeur v (candidat seul) de la zone iZon
//renvoie -1 si la valeur v n'est pas lue dans la zone
var
  c:0..8;
begin
  result:=-1;
  for c:=0 to 8 do
  if Caz^[IndCaz[iZon,c]].Cdt=[v]
  then begin
         result:=IndCaz[iZon,c];
         exit;
       end;
end;

function CasesContenantCdt(iZon:TiZon;v:TVal):TEnsCaz;
//renvoie l'ensemble des cases de la zone iZon qui contiennent le candidat v
var
  c:0..8;
begin
  result:=[];
  for c:=0 to 8 do
  if v in Caz^[IndCaz[iZon,c]].Cdt
  then result:=result+[IndCaz[iZon,c]];
end;

function NbMinCdt:byte;
//renvoie le plus petit nombre de candidats contenu dans au moins une case
var
  iCaz:TiCaz;
  Nbcdt:2..9;
begin
  result:=0;
  for NbCdt:=2 to 9 do
  for iCaz:=0 to 80 do
  if NbVal(Caz[iCaz].Cdt)=NbCdt
  then begin
         result:=NbCdt;
         exit;
       end;
end;

end.
